#include <Adafruit_NeoPixel.h>

// 定义麦克风输入的引脚
const int micPin = A0;
// 定义控制NeoPixel灯条的引脚
const int neoPixelPin = 9;
// NeoPixel灯条的LED数量
const int numPixels = 60;

// 创建Adafruit_NeoPixel类的实例
Adafruit_NeoPixel strip = Adafruit_NeoPixel(numPixels, neoPixelPin, NEO_GRB + NEO_KHZ800);

int lastBrightness = 0; // 存储上一次的亮度值

void setup() {
  // 初始化NeoPixel灯条
  strip.begin();
  strip.show(); // 将所有像素初始化为“关闭”状态
  // 初始化串行通信
  Serial.begin(9600);
}

// 生成彩虹颜色的函数
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void loop() {
  // 从麦克风读取值（假设是0到1023）
  int micValue = analogRead(micPin);
  // 将麦克风值映射到亮度级别（30到255），输入阈值0-1023
  int brightness = map(micValue, 0, 500, 255, 20);

  // 平滑亮度变化
  brightness = (lastBrightness + brightness) / 2;
  lastBrightness = brightness; // 更新上一次亮度值

  // 设置灯条中所有像素的颜色和亮度
  for (int i = 0; i < strip.numPixels(); i++) {
    // 使用彩虹颜色并根据声音强度调整亮度
    uint32_t color = Wheel((i * 256 / numPixels) & 255);
    uint8_t r = (uint8_t)(color >> 16),
            g = (uint8_t)(color >> 8),
            b = (uint8_t)color;
    r = (r * brightness) / 255;
    g = (g * brightness) / 255;
    b = (b * brightness) / 255;
    strip.setPixelColor(i, strip.Color(r, g, b));
  }
  strip.show();

  // 打印值进行调试
  Serial.print("麦克风值: ");
  Serial.print(micValue);
  Serial.print(" - 亮度: ");
  Serial.println(brightness);

  // 短暂延迟以降低读取频率
  delay(90);
}
